/*
 * test.c
 *
 *  Created on: Sep 27, 2010
 *      Author: christian
 */

#include <stdio.h>
#include <unistd.h>
#include <digitalSTROM/dsuid.h>

#include "dsm-api.h"
#include "dsm-api-const.h"

void BusChange_callback(void *arg, dsuid_t *id, int flag)
{
    int i;
    for (i = 0; i < sizeof(dsuid_t); ++i)
    {
        printf("%02x ", id->id[i]);
    }
    if (flag)
    {
        printf("left bus\n");
    }
    else
    {
        printf("joined bus\n");
    }
}

void LinkState_callback(void *arg, bool _state)
{
    if (_state) {
        printf("STATE: ACTIVE\n");
    } else {
        printf("STATE: ISOLATED\n");
    }
}

void EventDeviceAccessibility_on(uint8_t error_code, void *arg, dsuid_t sourceId, dsuid_t destinationId, uint16_t DeviceId, uint16_t ZoneId, uint16_t VendorId, uint16_t ProductId, uint16_t FunctionId, uint16_t version, dsuid_t dsuid)
{
    DSUID_STR(dsuid_dev_str);
    DSUID_STR(dsuid_dsm_str);
    dsuid_to_string(&dsuid, dsuid_dev_str);
    dsuid_to_string(&sourceId, dsuid_dsm_str);
    printf("Device 0x%s (DeviceId: %d in Zone: %d) became active on dSM %s",
           dsuid_dev_str, DeviceId, ZoneId, dsuid_dsm_str);
}

void EventDeviceAccessibility_off(uint8_t error_code, void *arg, dsuid_t sourceId, dsuid_t destinationId, uint16_t DeviceId, uint16_t ZoneId, uint16_t VendorId, uint16_t ProductId, uint16_t FunctionId, uint16_t version, dsuid_t dsuid)
{
    DSUID_STR(dsuid_dev_str);
    DSUID_STR(dsuid_dsm_str);
    dsuid_to_string(&dsuid, dsuid_dev_str);
    dsuid_to_string(&sourceId, dsuid_dsm_str);
    printf("Device 0x%s (DeviceId: %d in Zone: %d) became inactive on dSM %s",
           dsuid_dev_str, DeviceId, ZoneId, dsuid_dsm_str);
}

void ZoneCount_callback(uint8_t error_code, void *arg, dsuid_t sourceId, dsuid_t destinationId, uint8_t ZoneCount)
{
    DSUID_STR(dsuid_dsm_str);
    dsuid_to_string(&sourceId, dsuid_dsm_str);

    printf("dSM: %s has ZoneCount: %d (result: %d)\n", dsuid_dsm_str,
           ZoneCount, error_code);
}

void EventDeviceStatistics(uint8_t error_code, void *arg, dsuid_t sourceId, dsuid_t destinationId, uint16_t DeviceIndex, uint8_t Offset, uint16_t Value)
{
    printf("Device: %d, Offset %d: 0x%02x\n", DeviceIndex, Offset, Value);
}

void ZoneGroupActionRequest_action_call_scene_request(uint8_t error_code, void *arg, dsuid_t sourceId,
                                                      dsuid_t destinationId, uint16_t ZoneId, uint8_t GroupId,
                                                      uint16_t OriginDeviceId, uint8_t SceneId)
{
    DSUID_STR(dsuid_dsm_str);
    dsuid_to_string(&sourceId, dsuid_dsm_str);

    printf("Call scene %d in zone %d and group %d from dSM: %s\n", SceneId,
           ZoneId, GroupId, dsuid_dsm_str);
}


int main(int argc, char* argv[])
{
    int result;
    const char connection[] = "tcp://192.168.2.124:8442";
    DsmApiCallback_t callback;
    dsuid_t dsuid = { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16 };

    DsmApiHandle_t handle = DsmApiInitialize();

    result = DsmApiOpen(handle, connection, 0);

    LinkState_callback_t linkstate = &LinkState_callback;
    callback.function = &linkstate;
    callback.arg = NULL;
    DsmApiSetLinkStateCallback(handle, &callback, NULL);

    BusChange_callback_t buschange = &BusChange_callback;
    callback.function = (void*)buschange;
    callback.arg = NULL;
    DsmApiSetBusChangeCallback(handle, &callback, NULL);

    EventDeviceAccessibility_on_event_callback_t event_device_acc_on = &EventDeviceAccessibility_on;
    callback.function = (void*)event_device_acc_on;
    callback.arg = NULL;
    DsmApiSetCallback(handle, DS485_CONTAINER_EVENT, EVENT_DEVICE_ACCESSIBILITY, EVENT_DEVICE_ACCESSIBILITY_ON, &callback, NULL);

    EventDeviceAccessibility_off_event_callback_t event_device_acc_off = &EventDeviceAccessibility_off;
    callback.function = (void*)event_device_acc_off;
    callback.arg = NULL;
    DsmApiSetCallback(handle, DS485_CONTAINER_EVENT, EVENT_DEVICE_ACCESSIBILITY, EVENT_DEVICE_ACCESSIBILITY_OFF, &callback, NULL);

    ZoneCount_response_callback_t zone_count_callback = &ZoneCount_callback;
    callback.function = (void*)zone_count_callback;
    callback.arg = NULL;
    DsmApiSetCallback(handle, DS485_CONTAINER_RESPONSE, ZONE_COUNT, 0, &callback, NULL);

    EventDeviceStatistics_event_callback_t event_statistics_callback = &EventDeviceStatistics;
    callback.function = (void*)event_statistics_callback;
    callback.arg = NULL;
    DsmApiSetCallback(handle, DS485_CONTAINER_EVENT, EVENT_DEVICE_STATISTICS, 0, &callback, NULL);

    ZoneGroupActionRequest_action_call_scene_request_callback_t scene_callback = &ZoneGroupActionRequest_action_call_scene_request;
    callback.function = (void*)scene_callback;
    callback.arg = NULL;
    DsmApiSetCallback(handle, DS485_CONTAINER_REQUEST, ZONE_GROUP_ACTION_REQUEST, ZONE_GROUP_ACTION_REQUEST_ACTION_CALL_SCENE, &callback, NULL);

    DeviceStatisticsRequest_get(handle, dsuid, 65, 0xff);

    sleep(10);

    ZoneCount(handle, DSUID_BROADCAST, NULL);

    sleep(3600);
}
