global length_enum

class Parameter:
    def __init__(self, parameter):
        self.name = parameter.attrib["name"]
        self.type_size = int(parameter.attrib["type_size"])
        self.type_count = int(parameter.attrib["type_count"])
        self.offset = 0
        self.description = parameter.text if parameter.text else ""
        self.add_header = True if parameter.get("add_header", "false") == "true" else False
        self.dynamic = True if parameter.get("dynamic", "false") == "true" else False
        
    def setOffset(self, offset):
        self.offset = offset
        
    def getLength(self):
        if (self.dynamic):
            return 0
        else:
            return (self.type_size * self.type_count) / 8

    def getDynamicLength(self):
        if (self.dynamic):
            return self.name + "_Size"
        else:
            return None
    
    def lengthEnum(self, length_enum):
        if self.type_count > 1:
            const_name = self.name.upper() + "_LEN"
            while length_enum.has_key(const_name) and length_enum[const_name] != self.type_count:
                const_name += "_"
            length_enum[const_name] = self.type_count
    
class Error:
    def __init__(self, error):
        self.id = error.attrib["id"]
        self.value = error.attrib["value"]
        self.documentation = str(error.text)
        
    def getError(self):
        return self.id
    
    def getValue(self):
        return self.value
    
    def getEnumEntry(self):
        return str(self.id) + " = " + str(self.value) + ", /**< " + self.documentation +" */"

class Constant:
    def __init__(self, constant):
        self.name = constant.attrib["name"]
        self.value = constant.attrib["value"]
        self.documentation = str(constant.text) if constant.text else None

    def getEnumEntry(self):
        tmp = self.name + " = " + str(self.value) + ","
        if self.documentation:
            tmp += " /**< " + self.documentation +" */"
        return tmp

class CallGroup:
    def __init__(self, name, description=""):
        self.name = name
        self.description = description
        self.calls = list()
        self.events = list()
    
    def addCall(self, call):
        self.calls.append(call)
        
    def addEvent(self, event):
        self.events.append(event)

class ApiCall:
    def __init__(self, name, message_id, request_params, response_params, timeout, modifier_id=0, description=None, deprecated=False, long_message=False):
        self.name = name
        self.message_id = message_id
        self.request_params = request_params
        self.response_params = response_params
        self.modifier_id = modifier_id
        self.description = description
        self.timeout = timeout
        self.deprecated = deprecated
        self.long_message = long_message
        
        offset = 0
        if self.request_params:
            for i in self.request_params:
                i.setOffset(offset)
                offset += i.getLength()
            
        offset = 0
        if self.response_params:
            for i in self.response_params:
                i.setOffset(offset)
                offset += i.getLength()
    
    def fillLengthEnum(self, length_enum):
        if self.request_params:
            map(lambda x: x.lengthEnum(length_enum), self.request_params)
        if self.response_params:
            map(lambda x: x.lengthEnum(length_enum), self.response_params)
        
class ModifierId:
    def __init__(self, name, value, prefix):
        self.name = name
        self.value = value
        self.prefix = prefix
    
    def get_name(self):
        return self.name
        
    def get_value(self):
        return self.value
        
    def get_id(self):
        return self.prefix + self.name.upper()
        
class MessageId:
    def __init__(self, name, value):
        self.name = name
        
        first = True
        if (name[0:3] == "dSM"):
            name = "Dsm" + name[3:]
        if (name[0:3] == "dSF"):
            name = "Dsf" + name[3:]
        for c in name:
            if (first):
                self.id = c
                first = False
            else:
                if (c.isupper()):
                    self.id += "_" + c
                else:
                    self.id += c.upper()
        self.value = int(value, 16)
        self.prefix = self.id.upper() + "_"
        self.modifiers = list()
        self.timeout = 0
        
    def set_timeout(self, timeout):
        self.timeout = timeout
        
    def get_timeout(self):
        return self.timeout
        
    def get_enum_decl(self):
        return self.id + " = 0x%02x" % self.value
    
    def get_id(self):
        return self.id

    def add_modifier(self, modifier):
        if int(modifier.attrib["value"]) == 0:
            name = ""
        else:
            name = modifier.attrib["identifier"]
        self.modifiers.append(ModifierId(name,
                                         int(modifier.attrib["value"]), 
                                         self.prefix))
        return self.modifiers[len(self.modifiers) - 1]
        
    def get_modifier_enum(self):
        if len(self.modifiers) == 0:
            return ""
            
        out = "typedef enum {\n"
        out += "".join(map(lambda x: "    " + x.get_id() + " = " + str(x.get_value()) + ",\n", self.modifiers))
        out += "} " + self.name + "Modifiers_t;"
        return out
        
    def get_prefix(self):
        return self.prefix
        
    def get_name(self):
        return self.name
