/*
 This file is part of the digitalSTROM dsm-api library.

 Copyright (C) 2010-2014 digitalSTROM.org, Schlieren, Switzerland

 digitalSTROM dsm-api library is free software: you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 digitalSTROM dsm-api library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with digitalSTROM dsm-api library. If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * This is a helper tool for the dSS11 production test.
 *
 * It provides the means to run the functional tests for the ds485 hardware.
 */

#include <unistd.h>
#include <stdio.h>

#include <digitalSTROM/dsuid.h>

#include "dsm-api.h"
#include "dsm-api-const.h"

int main(int argc, char* argv[])
{
    int result;
    const char connection[] = "tcp://localhost:8442";
    dsuid_t busMembers[63];
    int device_list_size = 63;

    DsmApiHandle_t handle = DsmApiInitialize();
    if (handle == 0) {
        fprintf(stderr, "ERROR: could not initialize dsmapi handle\n");
        return 1;
    }

    result = DsmApiOpen(handle, connection, 0);
    if (result != 0) {
        fprintf(stderr, "ERROR %d: could not connect to ds485d %s\n",
                result, connection);
        DsmApiCleanup(handle);
        return 1;
    }

    result = DsmApiGetBusMembers(handle, busMembers, device_list_size);
    if (result < 0) {
        fprintf(stderr, "ERROR: could not get busmembers\n");
        DsmApiClose(handle);
        DsmApiCleanup(handle);
        return 1;
    }
    DsmApiClose(handle);
    DsmApiCleanup(handle);
    fprintf(stdout, "%d\n", result);
    return 0;
}
